# Contributing

#### Table of Contents

* [Code of Conduct](#code-of-conduct)
* [How to Contribute](#how-to-contribute)
  * [Reporting an Issue](#reporting-an-issue)
  * [Contributing Code](#contributing-code)
  * [Pull Requests](#pull-requests)
* [Style Guides](#style-guides)
  * [Git Commit Message](#git-commit-message)
  * [JavaScript](#javascript)
  * [Jasmine Tests](#jasmine-tests)
  * [Documentation](#documentation)
* [Definitions](#definitions)
* [Labels](#labels)
  * [General Labels](#general-labels)
  * [Contributor Labels](#contributor-labels)

## Code of Conduct

This project adheres to the Contributor Covenant [code of conduct](CODE_OF_CONDUCT.md).
By participating, you are expected to uphold this code.
Please report unacceptable behavior to [austinblatt@gmail.com](mailto:austinblatt@gmail.com).

## How to Contribute

### Reporting an Issue

Follow the steps to report a full and useful Issue report.

1. Check that an Issue has not previously been opened for this.
2. Create a new issue with a descriptive title.
3. Every new issue should have the `awaiting-review` tag. See [Labels](#labels) below
for descriptions of other potential tags.
4. Provide as much information as possible. Bugs should include a print out of any
error message(s) and the steps to reproduce them. Feature requests should explain
the reasoning behind adding the requested feature.
5. (Optional) Begin working on the Issue! You can assign the issue to
yourself and add the `in progress` label. Please read about labels below before doing this.

### Contributing Code

You want to contribute code? Awesome! Please read the steps below to
ensure your changes make it to production.

1. Ensure there is an Issue opened for it. Open an Issue if you need to.
2. Fork the repository.
3. Create a branch off master. Branch naming conventions, [issue number]_short_description
(ie. '2_contributing' is the name of the branch that this information was added).
2. Write the code! Follow this project's [Style Guides](#style-guides) listed below.
3. Tests. All tests must pass for the code to be merged into the production version.
  * If you fixed a bug, add tests to ensure this bug doesn't come back.
  * If you added a feature, add well thought-out descriptive tests to ensure its correctness.
4. Documentation... Ah the fun part. It would be great if you added documentation with
the code change, but this isn't always possible. If you do not add documentation, you must
open an Issue and add the tag `enhancement` so that we can document this at a later date.
5. Open a PR (following the requirements described below).

### Pull Requests

This code assumes you have followed the [Contributing Code](#contributing-code).

1. Squash all your work into a single commit that follows the
[style guide for commits](#git-commit-message).
2. Open a PR against master with the tag `awaiting review`

(Now the code needs a reviewer) The reviewer must be someone
with contributor status with the project.

3. The reviewer should switch the `awaiting review` label to `in progress`
and assign the PR to themselves.
4. The reviewer should look over the code, run the tests, test it manually.
If significant changes are needed to the PR, the reviewer should invite
more reviewers to ensure the changes will lead to an accepted PR.
5. Once a reviewer has accepted the code they should make a comment and
switch the label to `accepted`.
6. Merge the code into master.
7. Increment the version number as appropriate (using semantic versioning).
8. Publish the new version.
9. Close the issue once all these steps have been completed.

## Style Guides

### Git Commit Message

* Use the present tense ("Add feature" not "Added feature")
* Use the imperative mood in the title
("Move cursor to..." not "Moves cursor to...")
* The title line is limited to 50 characters. All subsequent lines are
limited to 72.
* The title should finish the following sentence "If applied, this commit will ..."
* The body should be used to describe why and what the commit does (usuall not how).

#### General format

```
(Issue Number) Short, descriptive title.

Describe how the program functioned before this commit. Describe why
this was not the desired function.

Describe what the program will do after this commit.
```

#### Example Commit

Here is an example commit.

```
(2) Add contributing guidelines to the repository

Before this commit, the project had no contributing guidelines. This
would make it difficult to work on with contributors because there
would not be a standard workflow.

This commit fixes that by adding a standard project workflow. This
will ensure that should this project have multiple contributors,
it will continue to be a robust library.

```

### JavaScript

### Jasmine Tests

### Documentation

## Definitions

* User - Any user of the razah library. Users may submit Issues using
the 'General Labels' described below. Users may also begin work on
Issues that have been labeled as 'accepted' by a Contributor. Users
are not authorized to review and/or merge Pull Requests.
* Contributor - Someone who has been granted contributor status on the
repository. Contributors can push code to the repositories and review Pull Requests
* PR - Pull Request.

## Labels

### General Labels

These labels can be added by anyone. Labels not listed here
are contributor only labels and should not be added or removed
by users.

| Tag | Description |
| --- | ----------- |
| `awaiting review` | Add this to every new Issue/PR. Contributors will use this to find and review new Issues and PRs. |
| `bug` | An error in the existing code base. Errors in documentation should also be listed under this tag. Security issues should also use this tag. |
| `feature` | A request for new functionality. |
| `enhancement` | A refactoring or optimization of existing code. Requests for additional documentation should use this tag. |
| `in progress` | You have began work on this Issue. Until it has an `accepted` tag, your work may not be added to the code base. Assign the issue to yourself when you add this tag. |
| `question` | A general question that is not a request for a code or documentation change. |

### Contributor Labels

These labels are used by the contributors to track the progress of
Issues and Pull Requests. Unless you are a Contributor to the project,
you should stick to the general labels described above.

| Tag | Description |
| --- | ----------- |
| `accepted` | For an Issue, this Issue has been accepted as a worthwhile change. For a PR, the reviewer has endorsed this code and the PR is waiting to be merged. |
| `controversial` | This Issue or PR contains a request or change that may or may not be a good addition to the library. |
| `duplicate` | This Issue or PR is or has been opened under another Issue or PR. |
| `invalid` | There is something wrong with the Issue or PR. See the comments for more information. |
| `needs information` | The contributors need more information about the Issue or PR in order to proceed. |
| `wont fix` | This is not in the scope of the library. |

