'use strict';

var _ = require('underscore');

var EndPointFactory = function(client) {
  var default_cb_wrapper = function(cb) {
    return function(data) {
      cb(data);
    };
  };


  this.static_endpoint = function(uri_path, callback_wrapper) {
    if(arguments.length === 1) {
      callback_wrapper = default_cb_wrapper;
    }

    return function(callback) {
      client.request(uri_path, callback_wrapper(callback));
    };
  };

  this.optional_ids_endpoint = function(uri_path, callback_wrapper) {
    if(arguments.length === 1) {
      callback_wrapper = default_cb_wrapper;
    }

    return function(ids, callback) {
      if(arguments.length === 1) {
        callback = ids;
        ids = null;
      }

      if(ids === null) {
        client.request(uri_path, callback_wrapper(callback));
      } else {
        client.request(uri_path, {ids: ids}, callback_wrapper(callback));
      }
    };
  };

  this.guild_id_endpoint = function(base_path, item_path, callback_wrapper) {
    if(arguments.length === 2) {
      callback_wrapper = default_cb_wrapper;
    }

    return function(guild_id, callback) {
      if(arguments.length === 1 && typeof(guild_id) === 'function' && _.has(client.getConfig(), 'guild_id')) {
        callback = guild_id;
        guild_id = client.getConfig().guild_id;
      } else {
        throw "Guild endpoints must be provided with a guild_id";
      }

      var request_path = base_path + '/' + guild_id + item_path;

      client.request(request_path, callback_wrapper(callback));
    };
  };
};

module.exports = EndPointFactory;

